package pwc.taxtech.atms.vat.service.impl;

import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Attachment;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dto.approval.ApprovalDto;
import pwc.taxtech.atms.dto.approval.ApprovalTask;
import pwc.taxtech.atms.exception.BadParameterException;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.PeriodApproveMapper;
import pwc.taxtech.atms.vat.dao.PeriodReportMapper;
import pwc.taxtech.atms.vat.entity.PeriodApprove;
import pwc.taxtech.atms.vat.entity.PeriodApproveExample;
import pwc.taxtech.atms.vat.entity.PeriodReport;
import pwc.taxtech.atms.vat.entity.PeriodReportExample;

import java.util.*;

public class ApprovalService {
    private static Logger logger = LoggerFactory.getLogger(ApprovalService.class);
    @Autowired
    RuntimeService runtimeService;
    @Autowired
    TaskService taskService;
    @Autowired
    PeriodApproveMapper periodApproveMapper;
    @Autowired
    PeriodReportMapper reportMapper;

    @Transactional
    public void startInstanceAndAssignee(ApprovalDto dto) {
        PeriodReportExample pre = new PeriodReportExample();
        pre.createCriteria().andProjectIdEqualTo(dto.getProjectId()).andPeriodEqualTo(dto.getPeriod());
        List<PeriodReport> currentReports = reportMapper.selectByExample(pre);
        MyAsserts.assertNotEmpty(currentReports, Exceptions.NOT_FOUND_REPORT_EXCEPTION);


        PeriodApprove pa = new PeriodApprove();
        pa.setId(UUID.randomUUID().toString());
        pa.setPeriod(dto.getPeriod());
        pa.setYear(dto.getYear());

        StringBuilder reportIds = new StringBuilder("");
        StringBuilder reportTemplateIds = new StringBuilder("");
        currentReports.forEach(m -> {
            reportIds.append(m.getId()).append(",");
            reportTemplateIds.append(m.getTemplateId()).append(",");
        });

        PeriodApproveExample pae = new PeriodApproveExample();
        pae.createCriteria().andProjectIdEqualTo(dto.getProjectId()).andPeriodEqualTo(dto.getPeriod())
                .andYearEqualTo(dto.getYear()).andReportIdsEqualTo(reportIds.toString())
                .andTemplateIdsEqualTo(reportTemplateIds.toString()).andStatusEqualTo(Constant.APPROVAL_COMMITTED);

        List<PeriodApprove> pas = periodApproveMapper.selectByExample(pae);
        MyAsserts.assertEmpty(pas, Exceptions.REPORT_HAS_COMMIT_EXCEPTION);

        ProcessInstance pi = runtimeService.startProcessInstanceByKey("approvalProcess");

        dto.setInstanceId(pi.getId());
        pa.setInstanceId(pi.getId());
        pa.setReportIds(reportIds.toString());
        pa.setTemplateIds(reportTemplateIds.toString());
        pa.setStatus(Constant.APPROVAL_COMMITTED);


        List<Task> tasks = taskService.createTaskQuery().taskAssignee(Constant.ASSIGNEE_ACCOUNTANT).processInstanceId(
                pi.getId()).list();
        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);

            Map<String, Object> map = new HashMap<>();
            map.put("committed", 0);

            taskService.complete(task.getId(), map);
            taskService.createAttachment("java.lang.String", task.getId(), task.getProcessInstanceId(),
                    "period_approval_uuid", pa.getId(), "");
            periodApproveMapper.insert(pa);

        } else {
            logger.warn("task must not null or size gt 1");
        }
    }

    public List<ApprovalTask> getTask(String assignee) {
        List<Task> tasks = taskService.createTaskQuery().taskAssignee(assignee).list();
        List<ApprovalTask> list = new ArrayList<>();
        for (Task task : tasks) {
            ApprovalTask t = new ApprovalTask();
            list.add(t.copyfrom(task));
        }
        return list;
    }

    @Transactional
    public void checkTask(String taskId, String decide) {
        Map<String, Object> map = new HashMap<>();
        PeriodApprove pa = new PeriodApprove();
        switch (decide) {
            case Constant.APPROVAL_AGREED:
                pa.setStatus(Constant.APPROVAL_AGREED);
                map.put("decide", 1);
                break;
            case Constant.APPROVAL_DISAGREED:
                pa.setStatus(Constant.APPROVAL_DISAGREED);
                map.put("decide", 0);
                break;
            default:
                throw new BadParameterException("not support decide param type");
        }
        List<Attachment> attachments = taskService.getTaskAttachments(taskId);
        if (attachments != null && attachments.size() == 1) {
            String uuid = attachments.get(0).getDescription();
            taskService.complete(taskId, map);

            pa.setId(uuid);
            periodApproveMapper.updateByPrimaryKeySelective(pa);
            PeriodApprove result=periodApproveMapper.selectByPrimaryKey(uuid);
            //save report on service
        } else {
            logger.warn("task must not null or size gt 1");
        }


    }

}
